#include    "Interrupts.h"

VECT_TBL	= 0xf780
RDR		= 0xffb5
SSR		= 0xffb4

        .h8300hn
        .file "isr.S"

;----------------------------------------------------------------------------
        .section .text
        .align 2
saveRegisters:
        mov.l   er0, @_registers		; save R0
        mov.l   #_registers, er0
        mov.l   er1, @(0x04, er0)		; save R1
        mov.l   er2, @(0x08, er0)		; save R2
        mov.l   er3, @(0x0c, er0)		; save R3
        mov.l   er4, @(0x10, er0)		; save R4
        mov.l   er5, @(0x14, er0)		; save R5
        mov.l   er6, @(0x18, er0)		; save R6

        mov.l   #0x0, er1			; zero the CCR location
        mov.l   er1, @(0x20, er0)

        mov.l   @er7, er1			; get off the stack(CCR)
        and.l   #0xffff0000, er1
        mov.l   er1, @(0x20, er0)		; save CCR

        mov.l   @er7, er1			; get off the stack(CCR & PC)
        and.l   #0x0000ffff, er1
        mov.l   er1, @(0x24, er0)		; save PC

        adds.l	#4, er7
        mov.l   er7, @(0x1c, er0)		; save R7

        mov.l   @_stub_sp, er7			; load _stub_stack
        mov.l   @_exception, er0
        jsr     @_handle_exception
        mov.l   er7, @_stub_sp			; save _stub_stack

;----------------------------------------------------------------------------
restoreRegisters:
        mov.l   #_registers, er0
        mov.l   @(0x1c, er0), er7			; restore R7

	mov.l	@(0x24, er0), er1			; PC
        mov.l   @(0x20, er0), er2			; CCR
	or.l    er2, er1				; CCR|PC

        push.l  er1

        mov.l   @(0x18, er0), er6			; restore R6
        mov.l   @(0x14, er0), er5			; restore R5
        mov.l   @(0x10, er0), er4			; restore R4
        mov.l   @(0x0c, er0), er3			; restore R3
        mov.l   @(0x08, er0), er2			; restore R2
        mov.l   @(0x04, er0), er1			; restore R1
        mov.b	#0x00, r0l
        mov.b	r0l, @_in_nmi				; in_nmi = 0        
        mov.l   @_registers, er0			; restore R0
        rte						; restore CCR & PC

;----------------------------------------------------------------------------
.global _INT_NMI
_INT_NMI:				/* NMI */
	mov.l   er0, @_work_area
        mov.l	#IntID_NMI, er0
        mov.l	er0, @_exception
        mov.b	@er0, r0l
        beq		NEXT
        pop.l	er0
        rte
NEXT:   mov.b	#0x01, r0l
        mov.b	r0l, @_in_nmi
        mov.l   @_work_area, er0
        bra		saveRegisters:16

;----------------------------------------------------------------------------
.global _INT_TRAP0
_INT_TRAP0:				/* BREAK */
	mov.l   er0, @_work_area
        mov.l	#IntID_TRAP0, er0
        mov.l	er0, @_exception
        mov.l   @_work_area, er0
        bra		saveRegisters:16


;----------------------------------------------------------------------------
.global _INT_TRAP1
_INT_TRAP1:				/* BIOS CALL */
	mov.l   er0, @_work_area
        mov.l	#IntID_TRAP1, er0
        mov.l	er0, @_exception
	mov.l   @_work_area, er0
        bra		saveRegisters:16

;----------------------------------------------------------------------------
.global _INT_TRAP2
_INT_TRAP2:
	mov.l   er0, @_work_area
        mov.l	#IntID_TRAP2, er0
        mov.l	er0, @_exception
	mov.l   @_work_area, er0
        bra		saveRegisters:16

;----------------------------------------------------------------------------
.global _INT_TRAP3
_INT_TRAP3:				/* BRAEK */
	mov.l   er0, @_work_area
        mov.l	#IntID_TRAP3, er0
        mov.l	er0, @_exception
	mov.l   @_work_area, er0
        bra		saveRegisters:16

;----------------------------------------------------------------------------
.global _INT_SLEEP
_INT_SLEEP:				/* BREAK */
	mov.l  er0, @_work_area
        ; clear IRR1.IRRDT
        mov.l  er1, @_exception		; use as work area
        mov.l  #0xfff6, er1		;IRR1
        mov.b  @er1, r0l		;
        and.b  #0x7f, r0l		;  IRRDT
        mov.b  r0l, @er1
        mov.l  @_exception, er1
        mov.l  #IntID_SLEEP, er0
        mov.l  er0, @_exception
        mov.l  @_work_area, er0
        bra		saveRegisters:16

;----------------------------------------------------------------------------
.global _INT_SCI3
_INT_SCI3:	                 /* SCI3 Rx */
	mov.l   er0, @_work_area
        mov.l	#IntID_SCI3, er0
        mov.l	er0, @_exception
	mov.l   @_work_area, er0
        bra		saveRegisters:16


;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
.global _INT_IRQ0
_INT_IRQ0:
        stc.w	ccr, @-er7
        subs	#2, er7
        push.l	er0
        mov.l	#IntID_IRQ0, er0
jmp_vect:
        mov.l	er0, @_exception
        push.l	er1
        shll.l	er0
        add.l	#VECT_TBL, er0
        mov.w	@er0, r1
        mov.w	@(0x0a, er7), r0
        mov.w	r0, e1
        mov.l	er1, @(0x08, er7)
        pop.l	er1
        pop.l	er0
        rte

;----------------------------------------------------------------------------
.global _INT_IRQ1
_INT_IRQ1:
        stc.w	ccr, @-er7
        subs	#2, er7
        push.l	er0
        mov.l	#IntID_IRQ1, er0
        bra		jmp_vect:16

;----------------------------------------------------------------------------
.global _INT_IRQ2
_INT_IRQ2:
        stc.w	ccr, @-er7
        subs	#2, er7
        push.l	er0
        mov.l	#IntID_IRQ2, er0
        bra		jmp_vect:16

;----------------------------------------------------------------------------
.global _INT_IRQ3
_INT_IRQ3:
        stc.w	ccr, @-er7
        subs	#2, er7
        push.l	er0
        mov.l	#IntID_IRQ3, er0
        bra		jmp_vect:16

;----------------------------------------------------------------------------
.global _INT_ABRK
_INT_ABRK:
        stc.w	ccr, @-er7
        subs	#2, er7
        push.l	er0
        mov.l	#IntID_ABRK, er0
        bra		jmp_vect:16

;----------------------------------------------------------------------------
.global _INT_WKP
_INT_WKP:
        stc.w	ccr, @-er7
        subs	#2, er7
        push.l	er0
        mov.l	#IntID_WKP, er0
        bra		jmp_vect:16

;----------------------------------------------------------------------------
.global _INT_TA
_INT_TA:
        stc.w	ccr, @-er7
        subs	#2, er7
        push.l	er0
        mov.l	#IntID_TA, er0
        bra		jmp_vect:16

;----------------------------------------------------------------------------
.global _INT_TW
_INT_TW:
        stc.w	ccr, @-er7
        subs	#2, er7
        push.l	er0
        mov.l	#IntID_TW, er0
        bra		jmp_vect:16

;----------------------------------------------------------------------------
.global _INT_TV
_INT_TV:
        stc.w	ccr, @-er7
        subs	#2, er7
        push.l	er0
        mov.l	#IntID_TV, er0
        bra		jmp_vect:16

;----------------------------------------------------------------------------
.global _INT_IIC2
_INT_IIC2:
        stc.w	ccr, @-er7
        subs	#2, er7
        push.l	er0
        mov.l	#IntID_IIC2, er0
        bra		jmp_vect:16

;----------------------------------------------------------------------------
.global _INT_ADI
_INT_ADI:
        stc.w	ccr, @-er7
        subs	#2, er7
        push.l	er0
        mov.l	#IntID_ADI, er0
        bra		jmp_vect:16

;----------------------------------------------------------------------------
.global _INT_Reserved
_INT_Reserved:
        stc.w	ccr, @-er7
        subs	#2, er7
        push.l	er0
        mov.l	#IntID_Reserved, er0
        bra		jmp_vect:16


